@@ -21,7 +21,6 @@
 #include "cal3d/bone.h"
 #include "cal3d/coreanimation.h"
 #include "cal3d/corebone.h"
-#include "cal3d/corekeyframe.h"
 #include "cal3d/corematerial.h"
 #include "cal3d/coremesh.h"
 #include "cal3d/coremodel.h"
@@ -29,7 +28,6 @@
 #include "cal3d/coreskeleton.h"
 #include "cal3d/coresubmesh.h"
 #include "cal3d/coresubmorphtarget.h"
-#include "cal3d/coretrack.h"
 #include "cal3d/datasource.h"
 #include "cal3d/error.h"
 #include "cal3d/hardwaremodel.h"
@@ -55,9 +53,8 @@
  /*****************************************************************************/
 /** \mainpage Cal3D API Reference
   * <center>
-  *   \image html ../../shared/cal3d.gif
   *   <p>
-  *   &copy; 2001 - 2003 Bruno Heidelberger
+  *   &copy; 2001 - 2006 Laurent Desmecht, Loic Dachary, Bruno Heidelberger
   *   <p>
   *     Answers to many frequently asked questions about Cal3D can be found
   *     in the <a href="./cal3dfaq.html">Cal3D FAQ</a>.
