@@ -24,6 +24,7 @@
 
 #include "cal3d/cal3d.h"
 #include "cal3d/cal3d_wrapper.h"
+using namespace cal3d;
 
 //****************************************************************************//
 // CalAnimation wrapper functions definition                                  //
@@ -73,7 +74,7 @@ Boolean CalAnimationAction_Execute(CalAnimationAction *self, float delayIn, floa
   return self->execute(delayIn, delayOut) ? True : False;
 }
 
-CalAnimationAction *CalAnimationAction_New(struct CalCoreAnimation *pCoreAnimation)
+CalAnimationAction *CalAnimationAction_New(CalCoreAnimation *pCoreAnimation)
 {
   return new CalAnimationAction(pCoreAnimation);
 }
@@ -83,6 +84,12 @@ Boolean CalAnimationAction_Update(CalAnimationAction *self, float deltaTime)
   return self->update(deltaTime) ? True : False;
 }
 
+void CalCoreAnimation_Scale(CalCoreAnimation *self, float factor)
+{
+  self->scale(factor);
+}
+
+
 //****************************************************************************//
 // CalAnimationCycle wrapper functions definition                             //
 //****************************************************************************//
@@ -211,19 +218,14 @@ void CalBone_SetCoreStateRecursive(CalBone *self)
 // CalCoreAnimation wrapper functions definition                              //
 //****************************************************************************//
 
-void CalCoreAnimation_AddCoreTrack(CalCoreAnimation *self, CalCoreTrack *pCoreTrack)
+CalCoreAnimation *CalCoreAnimation_New(CalCoreAnimation *self)
 {
-  self->addCoreTrack(pCoreTrack);
+    return explicitIncRef(new CalCoreAnimation());
 }
 
-void CalCoreAnimation_Delete(CalCoreAnimation *self)
+void CalCoreAnimation_Delete(CalCoreAnimation* self)
 {
-  delete self;
-}
-
-CalCoreTrack *CalCoreAnimation_GetCoreTrack(CalCoreAnimation *self, int coreBoneId)
-{
-  return self->getCoreTrack(coreBoneId);
+    explicitDecRef(self);
 }
 
 float CalCoreAnimation_GetDuration(CalCoreAnimation *self)
@@ -231,18 +233,6 @@ float CalCoreAnimation_GetDuration(CalCoreAnimation *self)
   return self->getDuration();
 }
 
-/*
-std::list<CalCoreTrack *>& CalCoreAnimation_GetListCoreTrack(CalCoreAnimation *self)
-{
-  return self->getListCoreTrack();
-}
-*/
-
-CalCoreAnimation *CalCoreAnimation_New(CalCoreAnimation *self)
-{
-  return new CalCoreAnimation();
-}
-
 void CalCoreAnimation_SetDuration(CalCoreAnimation *self, float duration)
 {
   self->setDuration(duration);
@@ -360,56 +350,36 @@ void CalCoreBone_SetUserData(CalCoreBone *self, CalUserData userData)
 }
 
 //****************************************************************************//
-// CalCoreKeyframe wrapper functions definition                               //
+// CalCoreMorphAnimation wrapper functions definition                         //
 //****************************************************************************//
 
-void CalCoreKeyframe_Delete(CalCoreKeyframe *self)
+CalCoreMorphAnimation *CalCoreMorphAnimation_New()
 {
-  delete self;
+    return new CalCoreMorphAnimation();
 }
 
-CalQuaternion *CalCoreKeyframe_GetRotation(CalCoreKeyframe *self)
+void CalCoreMorphAnimation_Delete(CalCoreMorphAnimation* self)
 {
-  return &(const_cast<CalQuaternion&>(self->getRotation()));
+    delete self;
 }
 
-float CalCoreKeyframe_GetTime(CalCoreKeyframe *self)
+Boolean CalCoreMorphAnimation_AddMorphTarget(struct CalCoreMorphAnimation* self, int meshID, int morphID)
 {
-  return self->getTime();
-}
-
-CalVector *CalCoreKeyframe_GetTranslation(CalCoreKeyframe *self)
-{
-  return &(const_cast<CalVector&>(self->getTranslation()));
-}
-
-CalCoreKeyframe *CalCoreKeyframe_New()
-{
-  return new CalCoreKeyframe();
-}
-
-void CalCoreKeyframe_SetRotation(CalCoreKeyframe *self, CalQuaternion *pRotation)
-{
-  self->setRotation(*pRotation);
-}
-
-void CalCoreKeyframe_SetTime(CalCoreKeyframe *self, float time)
-{
-  self->setTime(time);
-}
-
-void CalCoreKeyframe_SetTranslation(CalCoreKeyframe *self, CalVector *pTranslation)
-{
-  self->setTranslation(*pTranslation);
+    return self->addMorphTarget(meshID, morphID) ? True : False;
 }
 
 //****************************************************************************//
 // CalCoreMaterial wrapper functions definition                               //
 //****************************************************************************//
 
+CalCoreMaterial *CalCoreMaterial_New()
+{
+    return explicitIncRef(new CalCoreMaterial());
+}
+
 void CalCoreMaterial_Delete(CalCoreMaterial *self)
 {
-  delete self;
+    explicitDecRef(self);
 }
 
 /*
@@ -418,7 +388,6 @@ CalCoreMaterial::Color *CalCoreMaterial_GetAmbientColor(CalCoreMaterial *self)
   return &(self->getAmbientColor());
 }
 */
-
 /*
 CalCoreMaterial::Color *CalCoreMaterial_GetDiffuseColor(CalCoreMaterial *self)
 {
@@ -465,11 +434,6 @@ std::vector<Map>& CalCoreMaterial_GetVectorMap(CalCoreMaterial *self)
 }
 */
 
-CalCoreMaterial *CalCoreMaterial_New()
-{
-  return new CalCoreMaterial();
-}
-
 Boolean CalCoreMaterial_Reserve(CalCoreMaterial *self, int mapCount)
 {
   return self->reserve(mapCount) ? True : False;
@@ -522,15 +486,21 @@ void CalCoreMaterial_SetUserData(CalCoreMaterial *self, CalUserData userData)
 // CalCoreMesh wrapper functions definition                                   //
 //****************************************************************************//
 
+CalCoreMesh *CalCoreMesh_New()
+{
+    return explicitIncRef(new CalCoreMesh());
+}
+
+void CalCoreMesh_Delete(CalCoreMesh *self)
+{
+    explicitDecRef(self);
+}
+
 int CalCoreMesh_AddCoreSubmesh(CalCoreMesh *self, CalCoreSubmesh *pCoreSubmesh)
 {
   return self->addCoreSubmesh(pCoreSubmesh);
 }
 
-void CalCoreMesh_Delete(CalCoreMesh *self)
-{
-  delete self;
-}
 
 CalCoreSubmesh *CalCoreMesh_GetCoreSubmesh(CalCoreMesh *self, int id)
 {
@@ -549,9 +519,14 @@ std::vector<CalCoreSubmesh *>& CalCoreMesh_GetVectorCoreSubmesh(CalCoreMesh *sel
 }
 */
 
-CalCoreMesh *CalCoreMesh_New()
+void CalCoreMesh_Scale(CalCoreMesh *self,float factor)
 {
-  return new CalCoreMesh();
+  self->scale(factor);
+}
+
+int CalCoreMesh_AddAsMorphTarget(struct CalCoreMesh *self, struct CalCoreMesh *target)
+{
+  return self->addAsMorphTarget(target);
 }
 
 //****************************************************************************//
@@ -563,6 +538,11 @@ int CalCoreModel_AddCoreAnimation(CalCoreModel *self, CalCoreAnimation *pCoreAni
   return self->addCoreAnimation(pCoreAnimation);
 }
 
+int CalCoreModel_AddCoreMorphAnimation(CalCoreModel *self, CalCoreMorphAnimation *pCoreMorphAnimation)
+{
+  return self->addCoreMorphAnimation(pCoreMorphAnimation);
+}
+
 int CalCoreModel_AddCoreMaterial(CalCoreModel *self, CalCoreMaterial *pCoreMaterial)
 {
   return self->addCoreMaterial(pCoreMaterial);
@@ -628,6 +608,11 @@ CalUserData CalCoreModel_GetUserData(CalCoreModel *self)
   return self->getUserData();
 }
 
+int CalCoreModel_GetCoreMorphAnimationCount(CalCoreModel *self)
+{
+  return self->getCoreMorphAnimationCount();
+}
+
 int CalCoreModel_LoadCoreAnimation(CalCoreModel *self, char *strFilename)
 {
   return self->loadCoreAnimation(strFilename);
@@ -703,9 +688,14 @@ void CalCoreSkeleton_CalculateState(CalCoreSkeleton *self)
 }
 
 
+CalCoreSkeleton *CalCoreSkeleton_New()
+{
+    return explicitIncRef(new CalCoreSkeleton());
+}
+
 void CalCoreSkeleton_Delete(CalCoreSkeleton *self)
 {
-  delete self;
+    explicitDecRef(self);
 }
 
 CalCoreBone *CalCoreSkeleton_GetCoreBone(CalCoreSkeleton *self, int coreBoneId)
@@ -733,9 +723,19 @@ std::vector<CalCoreBone *>& CalCoreSkeleton_GetVectorCoreBone(CalCoreSkeleton *s
 }
 */
 
-CalCoreSkeleton *CalCoreSkeleton_New()
+void CalCoreSkeleton_Scale(CalCoreSkeleton *self,float factor)
 {
-  return new CalCoreSkeleton();
+	self->scale(factor);
+}
+
+void CalSkeleton_GetBoneBoundingBox(CalSkeleton *self, float *min, float *max)
+{
+	self->getBoneBoundingBox(min, max);
+}
+
+void CalSkeleton_CalculateBoundingBoxes(CalSkeleton *self)
+{
+	self->calculateBoundingBoxes();
 }
 
 //****************************************************************************//
@@ -867,52 +867,11 @@ Boolean CalCoreSubmesh_IsTangentsEnabled(CalCoreSubmesh *self, int mapId)
   return self->isTangentsEnabled(mapId) ? True : False;
 }
 
-Boolean CalCoreSubmesh_EnableTangents(struct CalCoreSubmesh *self, int mapId, bool enabled)
+Boolean CalCoreSubmesh_EnableTangents(CalCoreSubmesh *self, int mapId, bool enabled)
 {
   return self->enableTangents(mapId, enabled) ? True : False;
 }
 
-//****************************************************************************//
-// CalCoreTrack wrapper functions definition                                  //
-//****************************************************************************//
-
-Boolean CalCoreTrack_AddCoreKeyframe(CalCoreTrack *self, CalCoreKeyframe *pCoreKeyframe)
-{
-  return self->addCoreKeyframe(pCoreKeyframe) ? True : False;
-}
-
-void CalCoreTrack_Delete(CalCoreTrack *self)
-{
-  delete self;
-}
-
-int CalCoreTrack_GetCoreBoneId(CalCoreTrack *self)
-{
-  return self->getCoreBoneId();
-}
-
-/*
-std::map<float, CalCoreKeyframe *>& CalCoreTrack_GetMapCoreKeyframe(CalCoreTrack *self)
-{
-  return self->getMapCoreKeyframe();
-}
-*/
-
-Boolean CalCoreTrack_GetState(CalCoreTrack *self, float time, CalVector *pTranslation, CalQuaternion *pRotation)
-{
-  return self->getState(time, *pTranslation, *pRotation) ? True : False;
-}
-
-CalCoreTrack *CalCoreTrack_New()
-{
-  return new CalCoreTrack();
-}
-
-Boolean CalCoreTrack_SetCoreBoneId(CalCoreTrack *self, int coreBoneId)
-{
-  return self->setCoreBoneId(coreBoneId) ? True : False;
-}
-
 //****************************************************************************//
 // CalError wrapper functions definition                                      //
 //****************************************************************************//
@@ -966,22 +925,22 @@ void CalLoader_Delete(CalLoader *self)
 
 CalCoreAnimation *CalLoader_LoadCoreAnimation(CalLoader *self, char *strFilename)
 {
-  return self->loadCoreAnimation(strFilename);
+  return explicitIncRef(self->loadCoreAnimation(strFilename).get());
 }
 
 CalCoreMaterial *CalLoader_LoadCoreMaterial(CalLoader *self, char *strFilename)
 {
-  return self->loadCoreMaterial(strFilename);
+  return explicitIncRef(self->loadCoreMaterial(strFilename).get());
 }
 
 CalCoreMesh *CalLoader_LoadCoreMesh(CalLoader *self, char *strFilename)
 {
-  return self->loadCoreMesh(strFilename);
+  return explicitIncRef(self->loadCoreMesh(strFilename).get());
 }
 
 CalCoreSkeleton *CalLoader_LoadCoreSkeleton(CalLoader *self, char *strFilename)
 {
-  return self->loadCoreSkeleton(strFilename);
+  return explicitIncRef(self->loadCoreSkeleton(strFilename).get());
 }
 
 CalLoader *CalLoader_New()
@@ -1059,9 +1018,9 @@ void CalMixer_Delete(CalMixer *self)
   delete self;
 }
 
-Boolean CalMixer_ExecuteAction(CalMixer *self, int id, float delayIn, float delayOut)
+Boolean CalMixer_ExecuteAction(CalMixer *self, int id, float delayIn, float delayOut, enum Boolean autoLock)
 {
-  return self->executeAction(id, delayIn, delayOut, 1.0f) ? True : False;
+  return self->executeAction(id, delayIn, delayOut, 1.0f,  autoLock) ? True : False;
 }
 
 CalMixer *CalMixer_New(CalModel* pModel)
@@ -1079,6 +1038,45 @@ void CalMixer_UpdateSkeleton(CalMixer *self)
   self->updateSkeleton();
 }
 
+void CalMixer_RemoveAction(CalMixer *self,int id)
+{
+  self->removeAction(id);
+}
+
+//****************************************************************************//
+// CalMorphTargetMixer wrapper functions definition                           //
+//****************************************************************************//
+
+Boolean CalMorphTargetMixer_Blend(struct CalMorphTargetMixer *self, int id, float weight, float delay)
+{
+  return self->blend(id, weight, delay) ? True : False;
+}
+
+Boolean CalMorphTargetMixer_Clear(struct CalMorphTargetMixer *self, int id, float delay)
+{
+  return self->clear(id, delay) ? True : False;
+}
+
+void CalMorphTargetMixer_Delete(struct CalMorphTargetMixer *self)
+{
+  delete self;
+}
+
+struct CalMorphTargetMixer *CalMorphTargetMixer_New(struct CalModel *pModel)
+{
+  return new CalMorphTargetMixer(pModel);
+}
+
+void CalMorphTargetMixer_Update(struct CalMorphTargetMixer *self, float deltaTime)
+{
+  self->update(deltaTime);
+}
+
+int CalMorphTargetMixer_GetMorphTargetCount(struct CalMorphTargetMixer *self)
+{
+  return self->getMorphTargetCount();
+}
+
 //****************************************************************************//
 // CalModel wrapper functions definition                                      //
 //****************************************************************************//
@@ -1113,6 +1111,11 @@ CalMixer *CalModel_GetMixer(CalModel *self)
   return self->getMixer();
 }
 
+CalMorphTargetMixer *CalModel_GetMorphTargetMixer(CalModel *self)
+{
+  return self->getMorphTargetMixer();
+}
+
 CalPhysique *CalModel_GetPhysique(CalModel *self)
 {
   return self->getPhysique();
