@@ -12,30 +12,19 @@
 #include "config.h"
 #endif
 
-
 #include "cal3d/coreanimation.h"
 #include "cal3d/coretrack.h"
 
-
-CalCoreAnimation::CalCoreAnimation() : m_referenceCount(0)
+CalCoreAnimation::CalCoreAnimation()
 {
 }
 
 
 CalCoreAnimation::~CalCoreAnimation()
 {
-  // destroy all core tracks
-  while(!m_listCoreTrack.empty())
-  {
-    CalCoreTrack *pCoreTrack = m_listCoreTrack.front();
-    m_listCoreTrack.pop_front();
-
-    delete pCoreTrack;
-  }
 }
 
-
- /*****************************************************************************/
+/*****************************************************************************/
 /** Adds a core track.
   *
   * This function adds a core track to the core animation instance.
@@ -47,12 +36,14 @@ CalCoreAnimation::~CalCoreAnimation()
   *         \li \b false if an error happend
   *****************************************************************************/
 
-void CalCoreAnimation::addCoreTrack(CalCoreTrack *pCoreTrack)
+bool CalCoreAnimation::addCoreTrack(CalCoreTrack *pCoreTrack)
 {
   m_listCoreTrack.push_back(pCoreTrack);
+
+  return true;
 }
 
-/*****************************************************************************/
+ /*****************************************************************************/
 /** Provides access to a core track.
   *
   * This function returns the core track for a given bone ID.
@@ -72,7 +63,8 @@ CalCoreTrack *CalCoreAnimation::getCoreTrack(int coreBoneId)
   for(iteratorCoreTrack = m_listCoreTrack.begin(); iteratorCoreTrack != m_listCoreTrack.end(); ++iteratorCoreTrack)
   {
     // get the core bone
-    CalCoreTrack *pCoreTrack = *iteratorCoreTrack;
+    CalCoreTrack *pCoreTrack;
+    pCoreTrack = *iteratorCoreTrack;
 
     // check if we found the matching core bone
     if(pCoreTrack->getCoreBoneId() == coreBoneId) return pCoreTrack;
@@ -82,6 +74,19 @@ CalCoreTrack *CalCoreAnimation::getCoreTrack(int coreBoneId)
   return 0;
 }
 
+/*****************************************************************************/
+/** Gets the number of core tracks for this core animation.
+  *
+  * This function returns the number of core tracks used for this core animation.
+  *
+  * @return The number of core tracks
+  *****************************************************************************/
+
+unsigned int CalCoreAnimation::getTrackCount() const
+{
+	return m_listCoreTrack.size();
+}
+
  /*****************************************************************************/
 /** Returns the duration.
   *
@@ -90,25 +95,11 @@ CalCoreTrack *CalCoreAnimation::getCoreTrack(int coreBoneId)
   * @return The duration in seconds.
   *****************************************************************************/
 
-float CalCoreAnimation::getDuration()
+float CalCoreAnimation::getDuration() const
 {
   return m_duration;
 }
 
- /*****************************************************************************/
-/** Returns the core track list.
-  *
-  * This function returns the list that contains all core tracks of the core
-  * animation instance.
-  *
-  * @return A reference to the core track list.
-  *****************************************************************************/
-
-std::list<CalCoreTrack *>& CalCoreAnimation::getListCoreTrack()
-{
-  return m_listCoreTrack;
-}
-
  /*****************************************************************************/
 /** Sets the duration.
   *
@@ -137,11 +128,10 @@ void CalCoreAnimation::scale(float factor)
   std::list<CalCoreTrack *>::iterator iteratorCoreTrack;
   for(iteratorCoreTrack = m_listCoreTrack.begin(); iteratorCoreTrack != m_listCoreTrack.end(); ++iteratorCoreTrack)
   {
-    (*iteratorCoreTrack)->scale(factor);
+	  (*iteratorCoreTrack)->scale(factor);
   }
 }
 
-
  /*****************************************************************************/
 /** 
   * Set the name of the file in which the core animation is stored, if any.
@@ -164,7 +154,7 @@ void CalCoreAnimation::setFilename(const std::string& filename)
   *
   *****************************************************************************/
 
-const std::string& CalCoreAnimation::getFilename(void)
+const std::string& CalCoreAnimation::getFilename(void) const
 {
   return m_filename;
 }
@@ -191,39 +181,11 @@ void CalCoreAnimation::setName(const std::string& name)
   *
   *****************************************************************************/
 
-const std::string& CalCoreAnimation::getName(void)
+const std::string& CalCoreAnimation::getName(void) const
 {
   return m_name;
 }
 
-/*****************************************************************************/
-/** 
-  * Increment the reference counter the core animation.
-  *
-  *****************************************************************************/
-
-void CalCoreAnimation::incRef()
-{
-  m_referenceCount++;
-}
-
-/*****************************************************************************/
-/** 
-  * Decrement the reference counter the core animation.
-  *
-  * @return One of the following values:
-  *         \li \b true if there are nomore reference
-  *         \li \b false if there are another reference
-  *
-  *****************************************************************************/
-
-bool CalCoreAnimation::decRef()
-{
-  m_referenceCount--;
-  return (m_referenceCount <= 0);
-}
-
-
 /*****************************************************************************/
 /** 
   * Add a callback to the current list of callbacks for this CoreAnim.
@@ -242,7 +204,6 @@ void CalCoreAnimation::registerCallback(CalAnimationCallback *callback,float min
   m_listCallbacks.push_back(record);
 }
 
-
 /*****************************************************************************/
 /** 
   * Remove a callback from the current list of callbacks for this Anim.
@@ -264,5 +225,37 @@ void CalCoreAnimation::removeCallback(CalAnimationCallback *callback)
   }
 }
 
+/*****************************************************************************/
+/** Returns the core track list.
+  *
+  * This function returns the list that contains all core tracks of the core
+  * animation instance.
+  *
+  * @return A reference to the core track list.
+  *****************************************************************************/
+
+std::list<CalCoreTrack *>& CalCoreAnimation::getListCoreTrack()
+{
+  return m_listCoreTrack;
+}
+
+/*****************************************************************************/
+/** Returns the total number of core keyframes used for this animation.
+  *
+  * This function returns the total number of core keyframes used for this
+  * animation instance (i.e.: the sum of all core keyframes of all core tracks).
+  *
+  * @return A reference to the core track list.
+  *****************************************************************************/
+
+unsigned int CalCoreAnimation::getTotalNumberOfKeyframes() const
+{
+	unsigned int nbKeys = 0;
+	for (std::list<CalCoreTrack*>::const_iterator it = m_listCoreTrack.begin(); it != m_listCoreTrack.end(); ++it)
+	{
+		CalCoreTrack *track = *it;
+		nbKeys += track->getCoreKeyframeCount();
+	}
+	return nbKeys;
+}
 
-//****************************************************************************//
