@@ -27,6 +27,42 @@
 CalCoreKeyframe::CalCoreKeyframe()
   : m_time(0.0f)
 {
+}
+
+ /*****************************************************************************/
+/** Destructs the core keyframe instance.
+  *
+  * This function is the destructor of the core keyframe instance.
+  *****************************************************************************/
+
+CalCoreKeyframe::~CalCoreKeyframe()
+{
+}
+
+ /*****************************************************************************/
+/** Creates the core keyframe instance.
+  *
+  * This function creates the core keyframe instance.
+  *
+  * @return One of the following values:
+  *         \li \b true if successful
+  *         \li \b false if an error happend
+  *****************************************************************************/
+
+bool CalCoreKeyframe::create()
+{
+  return true;
+}
+
+ /*****************************************************************************/
+/** Destroys the core keyframe instance.
+  *
+  * This function destroys all data stored in the core keyframe instance and
+  * frees all allocated memory.
+  *****************************************************************************/
+
+void CalCoreKeyframe::destroy()
+{
 }
 
  /*****************************************************************************/
@@ -42,19 +78,6 @@ const CalQuaternion& CalCoreKeyframe::getRotation()
   return m_rotation;
 }
 
- /*****************************************************************************/
-/** Returns the time.
-  *
-  * This function returns the time of the core keyframe instance.
-  *
-  * @return The time in seconds.
-  *****************************************************************************/
-
-float CalCoreKeyframe::getTime()
-{
-  return m_time;
-}
-
  /*****************************************************************************/
 /** Returns the translation.
   *
