@@ -18,7 +18,7 @@
 
 
 CalCoreMaterial::CalCoreMaterial()
-  : m_userData(0), m_referenceCount(0)
+  : m_userData(0)
 {
 }
 
@@ -81,7 +81,8 @@ const std::string& CalCoreMaterial::getMapFilename(int mapId)
   if((mapId < 0) || (mapId >= (int)m_vectorMap.size()))
   {
     CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-    return Cal::strNull;
+    static const std::string strNull;
+    return strNull;
   }
 
   return m_vectorMap[mapId].strFilename;
@@ -351,32 +352,3 @@ void CalCoreMaterial::setUserData(Cal::UserData userData)
 {
   m_userData = userData;
 }
-
- /*****************************************************************************/
-/** 
-  * Increment the reference counter the core material.
-  *
-  *****************************************************************************/
-
-void CalCoreMaterial::incRef()
-{
-  m_referenceCount++;
-}
-
- /*****************************************************************************/
-/** 
-  * Decrement the reference counter the core material.
-  *
-  * @return One of the following values:
-  *         \li \b true if there are nomore reference
-  *         \li \b false if there are another reference
-  *
-  *****************************************************************************/
-
-bool CalCoreMaterial::decRef()
-{
-  m_referenceCount--;
-  return (m_referenceCount <= 0); 
-}
-
-//****************************************************************************//
