@@ -47,48 +47,6 @@ CalCoreModel::CalCoreModel(const std::string& name)
 
 CalCoreModel::~CalCoreModel()
 {
-  // destroy all core materials
-  std::vector<CalCoreMaterial *>::iterator iteratorCoreMaterial;
-  for(iteratorCoreMaterial = m_vectorCoreMaterial.begin(); iteratorCoreMaterial != m_vectorCoreMaterial.end(); ++iteratorCoreMaterial)
-  {
-	  if(*iteratorCoreMaterial) 
-	  {
-		  if((*iteratorCoreMaterial)->decRef())
-		  {
-			  delete (*iteratorCoreMaterial);
-		  }
-	  }
-  }
-  m_vectorCoreMaterial.clear();
-
-  // destroy all core meshes
-  std::vector<CalCoreMesh *>::iterator iteratorCoreMesh;
-  for(iteratorCoreMesh = m_vectorCoreMesh.begin(); iteratorCoreMesh != m_vectorCoreMesh.end(); ++iteratorCoreMesh)
-  {
-	  if(*iteratorCoreMesh) 
-	  {
-		  if((*iteratorCoreMesh)->decRef())
-		  {
-			  delete (*iteratorCoreMesh);
-		  }
-	  }
-  }
-  m_vectorCoreMesh.clear();
-
-  // destroy all core animations
-  std::vector<CalCoreAnimation *>::iterator iteratorCoreAnimation;
-  for(iteratorCoreAnimation = m_vectorCoreAnimation.begin(); iteratorCoreAnimation != m_vectorCoreAnimation.end(); ++iteratorCoreAnimation)
-  {
-	  if(*iteratorCoreAnimation) 
-	  {
-		  if((*iteratorCoreAnimation)->decRef())
-		  {
-			  delete (*iteratorCoreAnimation);
-		  }
-	  }
-  }
-  m_vectorCoreAnimation.clear();
-
   // destroy all core morph animations
   std::vector<CalCoreMorphAnimation *>::iterator iteratorCoreMorphAnimation;
   for(iteratorCoreMorphAnimation = m_vectorCoreMorphAnimation.begin(); iteratorCoreMorphAnimation != 
@@ -97,17 +55,6 @@ CalCoreModel::~CalCoreModel()
     delete (*iteratorCoreMorphAnimation);
   }
   m_vectorCoreMorphAnimation.clear();
-
-  if(m_pCoreSkeleton != 0)
-  {
-    if(m_pCoreSkeleton->decRef())
-	{
-		delete m_pCoreSkeleton;
-	}
-	m_pCoreSkeleton = 0;	
-  }
-
-  m_strName.erase();
 }
 
  /*****************************************************************************/
@@ -117,20 +64,13 @@ CalCoreModel::~CalCoreModel()
   *
   * @param pCoreAnimation A pointer to the core animation that should be added.
   *
-  * @return One of the following values:
-  *         \li the assigned animation \b ID of the added core animation
-  *         \li \b -1 if an error happend
+  * @return \li the assigned animation \b ID of the added core animation
   *****************************************************************************/
 
 int CalCoreModel::addCoreAnimation(CalCoreAnimation *pCoreAnimation)
 {
-  // get the id of the core animation
-  int animationId;
-  animationId = m_vectorCoreAnimation.size();
-
+  int animationId = m_vectorCoreAnimation.size();
   m_vectorCoreAnimation.push_back(pCoreAnimation);
-  pCoreAnimation->incRef();
-
   return animationId;
 }
 
@@ -173,11 +113,9 @@ int CalCoreModel::addCoreMorphAnimation(CalCoreMorphAnimation *pCoreMorphAnimati
 int CalCoreModel::addCoreMaterial(CalCoreMaterial *pCoreMaterial)
 {
   // get the id of the core material
-  int materialId;
-  materialId = m_vectorCoreMaterial.size();
+  int materialId = m_vectorCoreMaterial.size();
 
   m_vectorCoreMaterial.push_back(pCoreMaterial);
-  pCoreMaterial->incRef();
 
   return materialId;
 }
@@ -197,12 +135,8 @@ int CalCoreModel::addCoreMaterial(CalCoreMaterial *pCoreMaterial)
 int CalCoreModel::addCoreMesh(CalCoreMesh *pCoreMesh)
 {
   // get the id of the core mesh
-  int meshId;
-  meshId = m_vectorCoreMesh.size();
-
+  int meshId = m_vectorCoreMesh.size();
   m_vectorCoreMesh.push_back(pCoreMesh);
-  pCoreMesh->incRef();
-
   return meshId;
 }
 
@@ -248,7 +182,7 @@ CalCoreAnimation *CalCoreModel::getCoreAnimation(int coreAnimationId)
     return 0;
   }
 
-  return m_vectorCoreAnimation[coreAnimationId];
+  return m_vectorCoreAnimation[coreAnimationId].get();
 }
 
  /*****************************************************************************/
@@ -323,7 +257,7 @@ CalCoreMaterial *CalCoreModel::getCoreMaterial(int coreMaterialId)
     return 0;
   }
 
-  return m_vectorCoreMaterial[coreMaterialId];
+  return m_vectorCoreMaterial[coreMaterialId].get();
 }
 
  /*****************************************************************************/
@@ -400,7 +334,7 @@ CalCoreMesh *CalCoreModel::getCoreMesh(int coreMeshId)
     return 0;
   }
 
-  return m_vectorCoreMesh[coreMeshId];
+  return m_vectorCoreMesh[coreMeshId].get();
 }
 
  /*****************************************************************************/
@@ -428,7 +362,7 @@ int CalCoreModel::getCoreMeshCount()
 
 CalCoreSkeleton *CalCoreModel::getCoreSkeleton()
 {
-  return m_pCoreSkeleton;
+  return m_pCoreSkeleton.get();
 }
 
  /*****************************************************************************/
@@ -460,25 +394,19 @@ Cal::UserData CalCoreModel::getUserData()
 int CalCoreModel::loadCoreAnimation(const std::string& strFilename)
 {
   // the core skeleton has to be loaded already
-  if(m_pCoreSkeleton == 0)
+  if(!m_pCoreSkeleton)
   {
     CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
     return -1;
   }
 
   // load a new core animation
-  CalCoreAnimation *pCoreAnimation = CalLoader::loadCoreAnimation(strFilename);
-  if(pCoreAnimation == 0) return -1;
+  CalCoreAnimationPtr pCoreAnimation = CalLoader::loadCoreAnimation(strFilename, m_pCoreSkeleton.get());
+  if(!pCoreAnimation) return -1;
+  
 
   // add core animation to this core model
-  int animationId = addCoreAnimation(pCoreAnimation);
-  if(animationId == -1)
-  {
-    delete pCoreAnimation;
-    return -1;
-  }
-
-  return animationId;
+  return addCoreAnimation(pCoreAnimation.get());
 }
 
  /*****************************************************************************/
@@ -508,21 +436,20 @@ int CalCoreModel::loadCoreAnimation(const std::string& strFilename, const std::s
     id=(*it).second;
 
     // the core skeleton has to be loaded already
-    if(m_pCoreSkeleton == NULL)
+    if(!m_pCoreSkeleton)
     {
       CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
       return -1;
     }
-    if(m_vectorCoreAnimation[id] != NULL)
+    if(m_vectorCoreAnimation[id])
     {
       CalError::setLastError(CalError::INDEX_BUILD_FAILED, __FILE__, __LINE__);
       return -1;
     }
-    CalCoreAnimation *pCoreAnimation = CalLoader::loadCoreAnimation(strFilename);
-    if(pCoreAnimation == NULL) return -1;
+    CalCoreAnimationPtr pCoreAnimation = CalLoader::loadCoreAnimation(strFilename, m_pCoreSkeleton.get());
+    if(!pCoreAnimation) return -1;
     pCoreAnimation->setName(strAnimationName);
     m_vectorCoreAnimation[id] = pCoreAnimation;
-    pCoreAnimation->incRef();
   }
   else
   {
@@ -577,11 +504,7 @@ int CalCoreModel::unloadCoreAnimation(int coreAnimationId)
     return -1;
   }
 
-  if(m_vectorCoreAnimation[coreAnimationId]->decRef())
-  {
-	  delete m_vectorCoreAnimation[coreAnimationId];	  
-  }
-  m_vectorCoreAnimation[coreAnimationId] = 0;
+  m_vectorCoreAnimation[coreAnimationId] = CalCoreAnimationPtr (0);
 
   return coreAnimationId;
 }
@@ -602,25 +525,18 @@ int CalCoreModel::unloadCoreAnimation(int coreAnimationId)
 int CalCoreModel::loadCoreMaterial(const std::string& strFilename)
 {
   // the core skeleton has to be loaded already
-  if(m_pCoreSkeleton == 0)
+  if(!m_pCoreSkeleton)
   {
     CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
     return -1;
   }
 
   // load a new core material
-  CalCoreMaterial *pCoreMaterial = CalLoader::loadCoreMaterial(strFilename);
-  if(pCoreMaterial == 0) return -1;
+  CalCoreMaterialPtr pCoreMaterial = CalLoader::loadCoreMaterial(strFilename);
+  if(!pCoreMaterial) return -1;
 
   // add core material to this core model
-  int materialId = addCoreMaterial(pCoreMaterial);
-  if(materialId == -1)
-  {
-    delete pCoreMaterial;
-    return -1;
-  }
-
-  return materialId;
+  return addCoreMaterial(pCoreMaterial.get());
 }
 
  /*****************************************************************************/
@@ -650,21 +566,20 @@ int CalCoreModel::loadCoreMaterial(const std::string& strFilename, const std::st
     id=(*it).second;
 
     // the core skeleton has to be loaded already
-    if(m_pCoreSkeleton == NULL)
+    if(!m_pCoreSkeleton)
     {
       CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
       return -1;
     }
-    if(m_vectorCoreMaterial[id] != NULL)
+    if(m_vectorCoreMaterial[id])
     {
       CalError::setLastError(CalError::INDEX_BUILD_FAILED, __FILE__, __LINE__);
       return -1;
     }
-    CalCoreMaterial *pCoreMaterial = CalLoader::loadCoreMaterial(strFilename);
-    if(pCoreMaterial == NULL) return -1;
+    CalCoreMaterialPtr pCoreMaterial = CalLoader::loadCoreMaterial(strFilename);
+    if(!pCoreMaterial) return -1;
     pCoreMaterial->setName(strMaterialName);
     m_vectorCoreMaterial[id] = pCoreMaterial;
-    pCoreMaterial->incRef();
   }
   else
   {
@@ -719,11 +634,7 @@ int CalCoreModel::unloadCoreMaterial(int coreMaterialId)
     return -1;
   }
 
-  if(m_vectorCoreMaterial[coreMaterialId]->decRef())
-  {
-	  delete m_vectorCoreMaterial[coreMaterialId];	  
-  }
-  m_vectorCoreMaterial[coreMaterialId] = 0;
+  m_vectorCoreMaterial[coreMaterialId] = CalCoreMaterialPtr(0);
 
   return coreMaterialId;
 }
@@ -743,26 +654,18 @@ int CalCoreModel::unloadCoreMaterial(int coreMaterialId)
 int CalCoreModel::loadCoreMesh(const std::string& strFilename)
 {
   // the core skeleton has to be loaded already
-  if(m_pCoreSkeleton == 0)
+  if(!m_pCoreSkeleton)
   {
     CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
     return -1;
   }
 
   // load a new core mesh
-  CalCoreMesh *pCoreMesh = CalLoader::loadCoreMesh(strFilename);
-  if(pCoreMesh == 0) return -1;
+  CalCoreMeshPtr pCoreMesh = CalLoader::loadCoreMesh(strFilename);
+  if(!pCoreMesh) return -1;
 
   // add core mesh to this core model
-  int meshId;
-  meshId = addCoreMesh(pCoreMesh);
-  if(meshId == -1)
-  {
-    delete pCoreMesh;
-    return -1;
-  }
-
-  return meshId;
+  return addCoreMesh(pCoreMesh.get());
 }
 
  /*****************************************************************************/
@@ -792,21 +695,20 @@ int CalCoreModel::loadCoreMesh(const std::string& strFilename, const std::string
     id=(*it).second;
 
     // the core skeleton has to be loaded already
-    if(m_pCoreSkeleton == NULL)
+    if(!m_pCoreSkeleton)
     {
       CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
       return -1;
     }
-    if(m_vectorCoreMesh[id] != NULL)
+    if(m_vectorCoreMesh[id])
     {
       CalError::setLastError(CalError::INDEX_BUILD_FAILED, __FILE__, __LINE__);
       return -1;
     }
-    CalCoreMesh *pCoreMesh = CalLoader::loadCoreMesh(strFilename);
-    if(pCoreMesh == NULL) return -1;
+    CalCoreMeshPtr pCoreMesh = CalLoader::loadCoreMesh(strFilename);
+    if(!pCoreMesh) return -1;
     pCoreMesh->setName(strMeshName);
     m_vectorCoreMesh[id] = pCoreMesh;
-    pCoreMesh->incRef();
   }
   else
   {
@@ -862,11 +764,7 @@ int CalCoreModel::unloadCoreMesh(int coreMeshId)
     return -1;
   }
 
-  if(m_vectorCoreMesh[coreMeshId]->decRef())
-  {
-	  delete m_vectorCoreMesh[coreMeshId];	  
-  }
-  m_vectorCoreMesh[coreMeshId] = 0;
+  m_vectorCoreMesh[coreMeshId] = CalCoreMeshPtr(0);
 
   return coreMeshId;
 }
@@ -887,21 +785,9 @@ int CalCoreModel::unloadCoreMesh(int coreMeshId)
 
 bool CalCoreModel::loadCoreSkeleton(const std::string& strFilename)
 {
-  // destroy the current core skeleton
-  if(m_pCoreSkeleton != 0)
-  {
-    if(m_pCoreSkeleton->decRef())
-    {
-        delete m_pCoreSkeleton;
-    }        
-  }
-
   // load a new core skeleton
   m_pCoreSkeleton = CalLoader::loadCoreSkeleton(strFilename);
-  if(m_pCoreSkeleton == 0) return false;
-  m_pCoreSkeleton->incRef();
-
-  return true;
+  return bool(m_pCoreSkeleton);
 }
 
  /*****************************************************************************/
@@ -927,7 +813,7 @@ bool CalCoreModel::saveCoreAnimation(const std::string& strFilename, int coreAni
   }
 
   // save the core animation
-  if(!CalSaver::saveCoreAnimation(strFilename, m_vectorCoreAnimation[coreAnimationId]))
+  if(!CalSaver::saveCoreAnimation(strFilename, m_vectorCoreAnimation[coreAnimationId].get()))
   {
     return false;
   }
@@ -958,12 +844,7 @@ bool CalCoreModel::saveCoreMaterial(const std::string& strFilename, int coreMate
   }
 
   // save the core animation
-  if(!CalSaver::saveCoreMaterial(strFilename, m_vectorCoreMaterial[coreMaterialId]))
-  {
-    return false;
-  }
-
-  return true;
+  return CalSaver::saveCoreMaterial(strFilename, m_vectorCoreMaterial[coreMaterialId].get());
 }
 
  /*****************************************************************************/
@@ -989,12 +870,7 @@ bool CalCoreModel::saveCoreMesh(const std::string& strFilename, int coreMeshId)
   }
 
   // save the core animation
-  if(!CalSaver::saveCoreMesh(strFilename, m_vectorCoreMesh[coreMeshId]))
-  {
-    return false;
-  }
-
-  return true;
+  return CalSaver::saveCoreMesh(strFilename, m_vectorCoreMesh[coreMeshId].get());
 }
 
  /*****************************************************************************/
@@ -1012,19 +888,14 @@ bool CalCoreModel::saveCoreMesh(const std::string& strFilename, int coreMeshId)
 bool CalCoreModel::saveCoreSkeleton(const std::string& strFilename)
 {
   // check if we have a core skeleton in this code model
-  if(m_pCoreSkeleton == 0)
+  if(!m_pCoreSkeleton)
   {
     CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
     return false;
   }
 
   // save the core skeleton
-  if(!CalSaver::saveCoreSkeleton(strFilename, m_pCoreSkeleton))
-  {
-    return false;
-  }
-
-  return true;
+  return CalSaver::saveCoreSkeleton(strFilename, m_pCoreSkeleton.get());
 }
 
  /*****************************************************************************/
@@ -1080,17 +951,7 @@ void CalCoreModel::setCoreSkeleton(CalCoreSkeleton *pCoreSkeleton)
     CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
     return;
   }
-
-  // destroy a possible existing core skeleton
-  if(m_pCoreSkeleton != 0)
-  {
-	if(m_pCoreSkeleton->decRef())
-	{
-		delete m_pCoreSkeleton;
-	}
-  }  
   m_pCoreSkeleton = pCoreSkeleton;  
-  m_pCoreSkeleton->incRef();
 }
 
  /*****************************************************************************/
@@ -1118,7 +979,7 @@ void CalCoreModel::setUserData(Cal::UserData userData)
 void CalCoreModel::addBoneName(const std::string& strBoneName, int boneId)
 {
   //Make sure the skeleton has been loaded first
-  if (m_pCoreSkeleton != NULL)
+  if (m_pCoreSkeleton)
   {
     //Map the bone ID to the name
     m_pCoreSkeleton->mapCoreBoneName(boneId, strBoneName);
@@ -1138,7 +999,7 @@ void CalCoreModel::addBoneName(const std::string& strBoneName, int boneId)
 
 int CalCoreModel::getBoneId(const std::string& strBoneName)
 {
-  if (m_pCoreSkeleton != NULL)
+  if (m_pCoreSkeleton)
   {
     return m_pCoreSkeleton->getCoreBoneId(strBoneName);
   }
