@@ -16,7 +16,8 @@
 // Includes                                                                   //
 //****************************************************************************//
 
-#include "coresubmesh.h"
+#include "cal3d/coresubmesh.h"
+#include "cal3d/coresubmorphtarget.h"
 
  /*****************************************************************************/
 /** Constructs the core submesh instance.
@@ -25,9 +26,8 @@
   *****************************************************************************/
 
 CalCoreSubmesh::CalCoreSubmesh()
+  : m_coreMaterialThreadId(0), m_lodCount(0)
 {
-  m_coreMaterialThreadId = 0;
-  m_lodCount = 0;
 }
 
  /*****************************************************************************/
@@ -37,39 +37,6 @@ CalCoreSubmesh::CalCoreSubmesh()
   *****************************************************************************/
 
 CalCoreSubmesh::~CalCoreSubmesh()
-{
-  assert(m_vectorFace.empty());
-  assert(m_vectorVertex.empty());
-  assert(m_vectorPhysicalProperty.empty());
-  assert(m_vectorvectorTextureCoordinate.empty());
-  assert(m_vectorSpring.empty());
-  assert(m_vectorTangentsEnabled.empty());
-  assert(m_vectorvectorTangentSpace.empty());
-}
-
- /*****************************************************************************/
-/** Creates the core submesh instance.
-  *
-  * This function creates the core submesh instance.
-  *
-  * @return One of the following values:
-  *         \li \b true if successful
-  *         \li \b false if an error happend
-  *****************************************************************************/
-
-bool CalCoreSubmesh::create()
-{
-  return true;
-}
-
- /*****************************************************************************/
-/** Destroys the core submesh instance.
-  *
-  * This function destroys all data stored in the core submesh instance and
-  * frees all allocated memory.
-  *****************************************************************************/
-
-void CalCoreSubmesh::destroy()
 {
   // destroy all data
   m_vectorFace.clear();
@@ -79,6 +46,13 @@ void CalCoreSubmesh::destroy()
   m_vectorSpring.clear();
   m_vectorTangentsEnabled.clear();
   m_vectorvectorTangentSpace.clear();
+  // destroy all core sub morph targets
+  std::vector<CalCoreSubMorphTarget *>::iterator iteratorCoreSubMorphTarget;
+  for(iteratorCoreSubMorphTarget = m_vectorCoreSubMorphTarget.begin(); iteratorCoreSubMorphTarget != m_vectorCoreSubMorphTarget.end(); ++iteratorCoreSubMorphTarget)
+  {
+    delete (*iteratorCoreSubMorphTarget);
+  }
+  m_vectorCoreSubMorphTarget.clear();
 }
 
  /*****************************************************************************/
@@ -160,31 +134,32 @@ void CalCoreSubmesh::UpdateTangentVector(int v0, int v1, int v2, int mapId)
 {
   std::vector<CalCoreSubmesh::Vertex> &vvtx = getVectorVertex();
   std::vector<CalCoreSubmesh::TextureCoordinate> &vtex = m_vectorvectorTextureCoordinate[mapId];
-  
+
   // Step 1. Compute the approximate tangent vector.
   double du1 = vtex[v1].u - vtex[v0].u;
   double dv1 = vtex[v1].v - vtex[v0].v;
   double du2 = vtex[v2].u - vtex[v0].u;
   double dv2 = vtex[v2].v - vtex[v0].v;
-  
+
   double prod1 = (du1*dv2-dv1*du2);
   double prod2 = (du2*dv1-dv2*du1);
   if ((fabs(prod1) < 0.000001)||(fabs(prod2) < 0.000001)) return;
-  
+
   double x = dv2/prod1;
   double y = dv1/prod2;
 
   CalVector vec1 = vvtx[v1].position - vvtx[v0].position;
   CalVector vec2 = vvtx[v2].position - vvtx[v0].position;
   CalVector tangent = (vec1 * ((float)x)) + (vec2 * ((float)y));
-  
+
   // Step 2. Orthonormalize the tangent.
   double component = (tangent * vvtx[v0].normal);
   tangent -= (vvtx[v0].normal * ((float)component));
   tangent.normalize();
-  
+
   // Step 3: Add the estimated tangent to the overall estimate for the vertex.
 
+
   m_vectorvectorTangentSpace[mapId][v0].tangent+=tangent;
 }
 
@@ -203,8 +178,8 @@ bool CalCoreSubmesh::enableTangents(int mapId, bool enabled)
 
   if(!enabled)
   {
-	  m_vectorvectorTangentSpace[mapId].clear();
-	  return true;
+    m_vectorvectorTangentSpace[mapId].clear();
+    return true;
   }
 
   m_vectorvectorTangentSpace[mapId].reserve(m_vectorVertex.size());
@@ -213,8 +188,8 @@ bool CalCoreSubmesh::enableTangents(int mapId, bool enabled)
   int tangentId;
   for(tangentId=0;tangentId< (int)m_vectorvectorTangentSpace[mapId].size();tangentId++)
   {
-      m_vectorvectorTangentSpace[mapId][tangentId].tangent= CalVector(0.0f,0.0f,0.0f);
-	  m_vectorvectorTangentSpace[mapId][tangentId].crossFactor=1;
+    m_vectorvectorTangentSpace[mapId][tangentId].tangent= CalVector(0.0f,0.0f,0.0f);
+    m_vectorvectorTangentSpace[mapId][tangentId].crossFactor=1;
 
   }
 
@@ -222,14 +197,14 @@ bool CalCoreSubmesh::enableTangents(int mapId, bool enabled)
   int faceId;
   for(faceId=0;faceId<(int)m_vectorFace.size();faceId++)
   {
-	  UpdateTangentVector(m_vectorFace[faceId].vertexId[0],m_vectorFace[faceId].vertexId[1],m_vectorFace[faceId].vertexId[2],mapId);
-	  UpdateTangentVector(m_vectorFace[faceId].vertexId[1],m_vectorFace[faceId].vertexId[2],m_vectorFace[faceId].vertexId[0],mapId);
-	  UpdateTangentVector(m_vectorFace[faceId].vertexId[2],m_vectorFace[faceId].vertexId[0],m_vectorFace[faceId].vertexId[1],mapId);
+    UpdateTangentVector(m_vectorFace[faceId].vertexId[0],m_vectorFace[faceId].vertexId[1],m_vectorFace[faceId].vertexId[2],mapId);
+    UpdateTangentVector(m_vectorFace[faceId].vertexId[1],m_vectorFace[faceId].vertexId[2],m_vectorFace[faceId].vertexId[0],mapId);
+    UpdateTangentVector(m_vectorFace[faceId].vertexId[2],m_vectorFace[faceId].vertexId[0],m_vectorFace[faceId].vertexId[1],mapId);
   }
 
   for(tangentId=0;tangentId< (int)m_vectorvectorTangentSpace[mapId].size();tangentId++)
   {
-      m_vectorvectorTangentSpace[mapId][tangentId].tangent.normalize();
+    m_vectorvectorTangentSpace[mapId][tangentId].tangent.normalize();
   }
   
   return true;
@@ -377,7 +352,7 @@ bool CalCoreSubmesh::reserve(int vertexCount, int textureCoordinateCount, int fa
     m_vectorvectorTextureCoordinate[textureCoordinateId].reserve(vertexCount);
     m_vectorvectorTextureCoordinate[textureCoordinateId].resize(vertexCount);
 	
-	if (m_vectorTangentsEnabled[textureCoordinateId])
+    if (m_vectorTangentsEnabled[textureCoordinateId])
     {
       m_vectorvectorTangentSpace[textureCoordinateId].reserve(vertexCount);
       m_vectorvectorTangentSpace[textureCoordinateId].resize(vertexCount);
@@ -572,6 +547,138 @@ bool CalCoreSubmesh::setVertex(int vertexId, const Vertex& vertex)
   m_vectorVertex[vertexId] = vertex;
 
   return true;
+}
+
+ /*****************************************************************************/
+/** Adds a core sub morph target.
+  *
+  * This function adds a core sub morph target to the core sub mesh instance.
+  *
+  * @param pCoreSubMorphTarget A pointer to the core sub morph target that should be added.
+  *
+  * @return One of the following values:
+  *         \li the assigned sub morph target \b ID of the added core sub morph target
+  *         \li \b -1 if an error happend
+  *****************************************************************************/
+
+int CalCoreSubmesh::addCoreSubMorphTarget(CalCoreSubMorphTarget *pCoreSubMorphTarget)
+{
+  // get next sub morph target id
+  int subMorphTargetId;
+  subMorphTargetId = m_vectorCoreSubMorphTarget.size();
+
+  m_vectorCoreSubMorphTarget.push_back(pCoreSubMorphTarget);
+
+  return subMorphTargetId;
+}
+
+ /*****************************************************************************/
+/** Provides access to a core sub morph target.
+  *
+  * This function returns the core sub morph target with the given ID.
+  *
+  * @param id The ID of the core sub morph target that should be returned.
+  *
+  * @return One of the following values:
+  *         \li a pointer to the core sub morph target
+  *         \li \b 0 if an error happend
+  *****************************************************************************/
+
+CalCoreSubMorphTarget *CalCoreSubmesh::getCoreSubMorphTarget(int id)
+{
+  if((id < 0) || (id >= (int)m_vectorCoreSubMorphTarget.size()))
+  {
+    return 0;
+  }
+
+  return m_vectorCoreSubMorphTarget[id];
+}
+
+ /*****************************************************************************/
+/** Returns the number of core sub morph targets.
+  *
+  * This function returns the number of core sub morph targets in the core sub mesh
+  * instance.
+  *
+  * @return The number of core sub morph targets.
+  *****************************************************************************/
+
+int CalCoreSubmesh::getCoreSubMorphTargetCount()
+{
+  return m_vectorCoreSubMorphTarget.size();
+}
+
+ /*****************************************************************************/
+/** Returns the core sub morph target vector.
+  *
+  * This function returns the vector that contains all core sub morph target
+  *  of the core submesh instance.
+  *
+  * @return A reference to the core sub morph target vector.
+  *****************************************************************************/
+
+std::vector<CalCoreSubMorphTarget *>& CalCoreSubmesh::getVectorCoreSubMorphTarget()
+{
+  return m_vectorCoreSubMorphTarget;
+}
+
+ /*****************************************************************************/
+/** Scale the Submesh.
+  *
+  * This function rescale all the data that are in the core submesh instance.
+  *
+  * @param factor A float with the scale factor
+  *
+  *****************************************************************************/
+
+
+void CalCoreSubmesh::scale(float factor)
+{
+  // rescale all vertices
+
+  for(size_t vertexId = 0; vertexId < m_vectorVertex.size() ; vertexId++)
+  {
+    m_vectorVertex[vertexId].position*=factor;		
+  }
+
+  if(!m_vectorSpring.empty())
+  {
+
+    // There is a problem when we resize and that there is
+    // a spring system, I was unable to solve this
+    // problem, so I disable the spring system
+    // if the scale are too big
+
+    if( fabs(factor - 1.0f) > 0.10)
+    {
+      m_vectorSpring.clear();
+      m_vectorPhysicalProperty.clear();
+    }
+
+
+/*		
+		for(vertexId = 0; vertexId < m_vectorVertex.size() ; vertexId++)
+		{
+			//m_vectorPhysicalProperty[vertexId].weight *= factor;
+			m_vectorPhysicalProperty[vertexId].weight *= factor*factor;
+			//m_vectorPhysicalProperty[vertexId].weight *= 0.5f;
+		}
+
+
+		int springId;
+		for(springId = 0; springId < m_vectorVertex.size() ; springId++)
+		{
+			//m_vectorSpring[springId].idleLength*=factor;
+			CalVector distance = m_vectorVertex[m_vectorSpring[springId].vertexId[1]].position - m_vectorVertex[m_vectorSpring[springId].vertexId[0]].position;
+			
+			m_vectorSpring[springId].idleLength = distance.length();		
+		}
+
+   */
+  }
+
+	
+
 }
 
 //****************************************************************************//
