@@ -12,28 +12,34 @@
 #include "config.h"
 #endif
 
+//****************************************************************************//
+// Includes                                                                   //
+//****************************************************************************//
 
 #include "cal3d/coretrack.h"
 #include "cal3d/error.h"
 #include "cal3d/corekeyframe.h"
 
+ /*****************************************************************************/
+/** Constructs the core track instance.
+  *
+  * This function is the default constructor of the core track instance.
+  *****************************************************************************/
 
 CalCoreTrack::CalCoreTrack()
   : m_coreBoneId(-1)
 {
 }
 
+ /*****************************************************************************/
+/** Destructs the core track instance.
+  *
+  * This function is the destructor of the core track instance.
+  *****************************************************************************/
 
 CalCoreTrack::~CalCoreTrack()
 {
-  // destroy all core keyframes
-  for (unsigned i = 0; i < m_keyframes.size(); ++i)
-  {
-    delete m_keyframes[i];
-  }
-  m_keyframes.clear();
-
-  m_coreBoneId = -1;
+  assert(m_keyframes.empty());
 }
 
  /*****************************************************************************/
@@ -56,24 +62,42 @@ bool CalCoreTrack::addCoreKeyframe(CalCoreKeyframe *pCoreKeyframe)
     std::swap(m_keyframes[idx], m_keyframes[idx - 1]);
     --idx;
   }
-
   return true;
 }
 
  /*****************************************************************************/
-/** Returns the ID of the core bone.
+/** Creates the core track instance.
   *
-  * This function returns the ID of the core bone to which the core track
-  * instance is attached to.
+  * This function creates the core track instance.
   *
   * @return One of the following values:
-  *         \li the \b ID of the core bone
-  *         \li \b -1 if an error happend
+  *         \li \b true if successful
+  *         \li \b false if an error happend
   *****************************************************************************/
 
-int CalCoreTrack::getCoreBoneId()
+bool CalCoreTrack::create()
 {
-  return m_coreBoneId;
+  return true;
+}
+
+ /*****************************************************************************/
+/** Destroys the core track instance.
+  *
+  * This function destroys all data stored in the core track instance and frees
+  * all allocated memory.
+  *****************************************************************************/
+
+void CalCoreTrack::destroy()
+{
+	// destroy all core keyframes
+	for (unsigned int i = 0; i < m_keyframes.size(); ++i)
+	{
+		m_keyframes[i]->destroy();
+		delete m_keyframes[i];
+	}
+  m_keyframes.clear();
+
+  m_coreBoneId = -1;
 }
 
  /*****************************************************************************/
@@ -151,6 +175,12 @@ std::vector<CalCoreKeyframe*>::iterator CalCoreTrack::getUpperBound(float time)
 
   int lowerBound = 0;
   int upperBound = m_keyframes.size()-1;
+	//static int aa = 0;
+
+	//upperBound += aa;
+	//upperBound %= m_keyframes.size();
+	//aa++;
+	//time = m_keyframes[upperBound]->getTime();
 
   while(lowerBound<upperBound-1)
   {
@@ -164,6 +194,7 @@ std::vector<CalCoreKeyframe*>::iterator CalCoreTrack::getUpperBound(float time)
 	  {
 		  upperBound=middle;
 	  }
+		//break;
   }
 
   return m_keyframes.begin() + upperBound;
