@@ -88,6 +88,45 @@ CalMixer::~CalMixer()
   m_pModel = 0;
 }
 
+///
+/// Examines the given animation and if the first and last keyframe of a given track
+/// do not match up, the first key frame is duplicated and added to the end of the track
+/// to ensure smooth looping.
+///
+static void addExtraKeyframeForLoopedAnim(CalCoreAnimation* pCoreAnimation)
+{
+	std::list<CalCoreTrack*>& listCoreTrack = pCoreAnimation->getListCoreTrack();
+
+   if(listCoreTrack.size() == 0)
+		 return;
+
+	CalCoreTrack *coreTrack = listCoreTrack.front();
+  if(coreTrack == 0)
+		 return;
+
+	CalCoreKeyframe *lastKeyframe = coreTrack->getCoreKeyframe(coreTrack->getCoreKeyframeCount()-1);
+	if(lastKeyframe == 0)
+		 return;
+
+	if(lastKeyframe->getTime() < pCoreAnimation->getDuration())
+	{
+		std::list<CalCoreTrack *>::iterator itr;
+    for(itr=listCoreTrack.begin();itr!=listCoreTrack.end();++itr)
+		{
+			coreTrack = *itr;
+
+			CalCoreKeyframe *firstKeyframe = coreTrack->getCoreKeyframe(0);
+      CalCoreKeyframe *newKeyframe = new CalCoreKeyframe();
+
+      newKeyframe->setTranslation(firstKeyframe->getTranslation());
+      newKeyframe->setRotation(firstKeyframe->getRotation());
+      newKeyframe->setTime(pCoreAnimation->getDuration());
+
+      coreTrack->addCoreKeyframe(newKeyframe);
+		}
+	}
+}
+
  /*****************************************************************************/
 /** Interpolates the weight of an animation cycle.
   *
@@ -125,34 +164,9 @@ bool CalMixer::blendCycle(int id, float weight, float delay)
     CalCoreAnimation *pCoreAnimation = m_pModel->getCoreModel()->getCoreAnimation(id);
     if(pCoreAnimation == 0) return false;
 
-	std::list<CalCoreTrack*>& listCoreTrack = pCoreAnimation->getListCoreTrack();
-
-    if(listCoreTrack.size() == 0) return false;
-    
-	CalCoreTrack *coreTrack = listCoreTrack.front();
-    if(coreTrack == 0) return false;
-
-	CalCoreKeyframe *lastKeyframe = coreTrack->getCoreKeyframe(coreTrack->getCoreKeyframeCount()-1);
-    if(lastKeyframe == 0) return false;
-
-	if(lastKeyframe->getTime()<pCoreAnimation->getDuration())
-	{
-
-		std::list<CalCoreTrack *>::iterator itr;
-		for(itr=listCoreTrack.begin();itr!=listCoreTrack.end();++itr)
-		{
-			coreTrack = *itr;
-
-            CalCoreKeyframe *firstKeyframe = coreTrack->getCoreKeyframe(0);
-			CalCoreKeyframe *newKeyframe = new CalCoreKeyframe();
-
-			newKeyframe->setTranslation(firstKeyframe->getTranslation());
-            newKeyframe->setRotation(firstKeyframe->getRotation());
-            newKeyframe->setTime(pCoreAnimation->getDuration());
-
-			coreTrack->addCoreKeyframe(newKeyframe);
-		}
-	}
+    // Ensure that the animation's first and last key frame match for proper
+    // looping.
+    ::addExtraKeyframeForLoopedAnim(pCoreAnimation);
 
     // allocate a new animation cycle instance
     CalAnimationCycle *pAnimationCycle = new CalAnimationCycle(pCoreAnimation);
@@ -352,7 +366,7 @@ void CalMixer::updateAnimation(float deltaTime)
   else
   {
     m_animationTime += deltaTime * m_timeFactor;
-    if(m_animationTime >= m_animationDuration)
+    if(m_animationTime >= m_animationDuration || m_animationTime<0)
     {
       m_animationTime = (float) fmod(m_animationTime, m_animationDuration);
     }
@@ -370,7 +384,7 @@ void CalMixer::updateAnimation(float deltaTime)
     // update and check if animation action is still active
     if((*iteratorAnimationAction)->update(deltaTime))
     {
-      (*iteratorAnimationAction)->checkCallbacks(m_animationTime,m_pModel);
+      (*iteratorAnimationAction)->checkCallbacks((*iteratorAnimationAction)->getTime(),m_pModel);
       ++iteratorAnimationAction;
     }
     else
@@ -534,7 +548,6 @@ void CalMixer::updateSkeleton()
   * @return The animation time in seconds.
   *****************************************************************************/
 
-
 float CalMixer::getAnimationTime()
 {
 	return m_animationTime;
@@ -548,13 +561,11 @@ float CalMixer::getAnimationTime()
   * @return The animation duration in seconds.
   *****************************************************************************/
 
-
 float CalMixer::getAnimationDuration()
 {
 	return m_animationDuration;
 }
 
-
 /*****************************************************************************/
 /** Sets the animation time.
   *
@@ -562,7 +573,6 @@ float CalMixer::getAnimationDuration()
   *
   *****************************************************************************/
 
-
 void CalMixer::setAnimationTime(float animationTime)
 {
 	m_animationTime=animationTime;
@@ -576,7 +586,6 @@ void CalMixer::setAnimationTime(float animationTime)
   *
   *****************************************************************************/
 
-
 void CalMixer::setTimeFactor(float timeFactor)
 {
     m_timeFactor = timeFactor;
@@ -594,4 +603,52 @@ float CalMixer::getTimeFactor()
     return m_timeFactor;
 }
 
+/*****************************************************************************/
+/** Get the model.
+  * 
+  * This function return the CalModel of the mixer instance.
+  *
+  *****************************************************************************/
+
+CalModel *CalMixer::getCalModel() 
+{ 
+    return m_pModel; 
+} 
+
+/*****************************************************************************/
+/** Get the animation vector.
+  * 
+  * This function return the animation vector of the mixer instance.
+  *
+  *****************************************************************************/
+
+std::vector<CalAnimation *> & CalMixer::getAnimationVector() 
+{ 
+    return m_vectorAnimation; 
+} 
+
+/*****************************************************************************/
+/** Get the list of the action animation.
+  * 
+  * This function return the list of the action animation of the mixer instance.
+  *
+  *****************************************************************************/
+
+std::list<CalAnimationAction *> & CalMixer::getAnimationActionList() 
+{ 
+    return m_listAnimationAction; 
+}
+
+/*****************************************************************************/
+/** Get the list of the cycle animation.
+  * 
+  * This function return the list of the cycle animation of the mixer instance.
+  *
+  *****************************************************************************/
+
+std::list<CalAnimationCycle *> & CalMixer::getAnimationCycle() 
+{ 
+    return m_listAnimationCycle; 
+} 
+
 //****************************************************************************//
