@@ -24,13 +24,15 @@
 #include "cal3d/coreskeleton.h"
 #include "cal3d/corebone.h"
 #include "cal3d/coreanimation.h"
-#include "cal3d/coretrack.h"
-#include "cal3d/corekeyframe.h"
 #include "cal3d/coremesh.h"
 #include "cal3d/coresubmesh.h"
 #include "cal3d/corematerial.h"
+#include "cal3d/corekeyframe.h"
+#include "cal3d/coretrack.h"
 #include "cal3d/tinyxml.h"
 
+using namespace cal3d;
+
  /*****************************************************************************/
 /** Saves a core animation instance.
   *
@@ -81,16 +83,16 @@ bool CalSaver::saveCoreAnimation(const std::string& strFilename, CalCoreAnimatio
     return false;
   }
 
-  // get core track list
-  std::list<CalCoreTrack *>& listCoreTrack = pCoreAnimation->getListCoreTrack();
-
   // write the number of tracks
-  if(!CalPlatform::writeInteger(file, listCoreTrack.size()))
+  if(!CalPlatform::writeInteger(file, pCoreAnimation->getTrackCount()))
   {
     CalError::setLastError(CalError::FILE_WRITING_FAILED, __FILE__, __LINE__, strFilename);
     return 0;
   }
 
+  // get core track list
+  std::list<CalCoreTrack *>& listCoreTrack = pCoreAnimation->getListCoreTrack();
+
   // write all core bones
   std::list<CalCoreTrack *>::iterator iteratorCoreTrack;
   for(iteratorCoreTrack = listCoreTrack.begin(); iteratorCoreTrack != listCoreTrack.end(); ++iteratorCoreTrack)
@@ -686,7 +688,7 @@ bool CalSaver::saveCoreTrack(std::ofstream& file, const std::string& strFilename
     return false;
   }
 
-  // read the number of keyframes
+  // write the number of keyframes
   if(!CalPlatform::writeInteger(file, pCoreTrack->getCoreKeyframeCount()))
   {
     CalError::setLastError(CalError::FILE_WRITING_FAILED, __FILE__, __LINE__, strFilename);
@@ -697,8 +699,9 @@ bool CalSaver::saveCoreTrack(std::ofstream& file, const std::string& strFilename
   for(int i = 0; i < pCoreTrack->getCoreKeyframeCount(); ++i)
   {
     // save the core keyframe
-    if(!saveCoreKeyframe(file, strFilename, pCoreTrack->getCoreKeyframe(i)))
-    {
+		bool res = saveCoreKeyframe(file, strFilename, pCoreTrack->getCoreKeyframe(i));
+
+		if (!res) {
       return false;
     }
   }
@@ -858,48 +861,44 @@ bool CalSaver::saveXmlCoreSkeleton(const std::string& strFilename, CalCoreSkelet
 
 bool CalSaver::saveXmlCoreAnimation(const std::string& strFilename, CalCoreAnimation *pCoreAnimation)
 {
+	std::stringstream str;
 
-    std::stringstream str;
-
-    TiXmlDocument doc(strFilename);
-
-    TiXmlElement animation("ANIMATION");
-    //animation.SetAttribute("MAGIC",Cal::ANIMATION_XMLFILE_MAGIC);
-    animation.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
+	TiXmlDocument doc(strFilename);
 
+	TiXmlElement animation("ANIMATION");
+	//animation.SetAttribute("MAGIC",Cal::ANIMATION_XMLFILE_MAGIC);
+	animation.SetAttribute("VERSION",Cal::LIBRARY_VERSION);
 
 	str.str("");
-    str << pCoreAnimation->getDuration();	
+	str << pCoreAnimation->getDuration();	
 	animation.SetAttribute("DURATION",str.str());
-	
+	animation.SetAttribute("NUMTRACKS", pCoreAnimation->getTrackCount());
+
 	// get core track list
-    std::list<CalCoreTrack *>& listCoreTrack = pCoreAnimation->getListCoreTrack();
+	std::list<CalCoreTrack *>& listCoreTrack = pCoreAnimation->getListCoreTrack();
 
-	animation.SetAttribute("NUMTRACKS",listCoreTrack.size());
-  
-
-    // write all core bones
-    std::list<CalCoreTrack *>::iterator iteratorCoreTrack;
-    for(iteratorCoreTrack = listCoreTrack.begin(); iteratorCoreTrack != listCoreTrack.end(); ++iteratorCoreTrack)
+	// write all core bones
+	std::list<CalCoreTrack *>::iterator iteratorCoreTrack;
+	for(iteratorCoreTrack = listCoreTrack.begin(); iteratorCoreTrack != listCoreTrack.end(); ++iteratorCoreTrack)
 	{
 		CalCoreTrack *pCoreTrack=*iteratorCoreTrack;
 
 		TiXmlElement track("TRACK");
 		track.SetAttribute("BONEID",pCoreTrack->getCoreBoneId());
-		
+
 		track.SetAttribute("NUMKEYFRAMES",pCoreTrack->getCoreKeyframeCount());
 
 		// save all core keyframes
-        for (int i = 0; i < pCoreTrack->getCoreKeyframeCount(); ++i)
+		for (int i = 0; i < pCoreTrack->getCoreKeyframeCount(); ++i)
 		{
 			CalCoreKeyframe *pCoreKeyframe=pCoreTrack->getCoreKeyframe(i);
 
 			TiXmlElement keyframe("KEYFRAME");
 
 			str.str("");
-            str << pCoreKeyframe->getTime();	        
+			str << pCoreKeyframe->getTime();	        
 			keyframe.SetAttribute("TIME",str.str());
-			
+
 			TiXmlElement translation("TRANSLATION");
 			const CalVector& translationVector = pCoreKeyframe->getTranslation();
 
@@ -933,14 +932,14 @@ bool CalSaver::saveXmlCoreAnimation(const std::string& strFilename, CalCoreAnima
 	}
 
 	doc.InsertEndChild(animation);
-	
-    if(!doc.SaveFile())
+
+	if(!doc.SaveFile())
 	{
-	  CalError::setLastError(CalError::FILE_WRITING_FAILED, __FILE__, __LINE__, strFilename);
-      return false;
+		CalError::setLastError(CalError::FILE_WRITING_FAILED, __FILE__, __LINE__, strFilename);
+		return false;
 	} 
 
-  return true;
+	return true;
 }
 
 
