@@ -84,9 +84,9 @@ CalSkeleton::~CalSkeleton()
 void CalSkeleton::calculateState()
 {
   // calculate all bone states of the skeleton
-  std::list<int>& listRootCoreBoneId = m_pCoreSkeleton->getListRootCoreBoneId();
+  std::vector<int>& listRootCoreBoneId = m_pCoreSkeleton->getVectorRootCoreBoneId();
 
-  std::list<int>::iterator iteratorRootBoneId;
+  std::vector<int>::iterator iteratorRootBoneId;
   for(iteratorRootBoneId = listRootCoreBoneId.begin(); iteratorRootBoneId != listRootCoreBoneId.end(); ++iteratorRootBoneId)
   {
     m_vectorBone[*iteratorRootBoneId]->calculateState();
